#include "../Headers/music.h"

/*
 *   SUPER MARIO THEME
 */
//Array for the notes, description in attached xlsx
volatile const uint16_t mario_notes[MARIO_LEN] = {30256, 30256, 0, 30256, 0, 38131, 30256, 0, 25444, 0, 0, 50890, 0, 0, 38131, 0, 0, 50890, 0, 60513, 0, 0, 45350, 0, 40403, 0, 42803, 45350, 0, 50890, 0, 30256, 0, 25444, 0, 22675, 0, 28570, 25444, 0, 30256, 0, 38131, 33984, 20201, 0, 38131, 0, 0, 50890, 0, 60513, 0, 0, 45350, 0, 40403, 0, 42803, 45350, 0, 50890, 0, 30256, 0, 25444, 22675, 0, 28570, 25444, 0, 30256, 0, 38131, 33984, 20201, 0, 0, 25444, 24023, 28570, 32076, 0, 30256, 0, 48047, 45350, 38131, 0, 45350, 38131, 33984, 0, 25444, 26971, 28570, 32076, 0, 30256, 0, 19110, 0, 19110, 19110, 0, 0, 0, 25444, 26971, 28570, 32076, 0, 30256, 0, 48047, 45350, 38131, 0, 45350, 38131, 33984, 0, 30256, 0, 0, 33984, 0, 38131, 0, 0, 0};
//Array for the beats, description in attached xlsx
volatile const uint8_t mario_beats[MARIO_LEN] = {6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 4, 6, 6, 4, 6, 6, 6, 6, 4, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 4, 6, 6, 6, 6, 4, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 4, 4, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 4, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 4, 4, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 4, 6, 6, 6, 6, 4, 6, 6, 4, 2};

/*
 *   BOCI, BOCI, TARKA (Hungarian children's song)
 */
volatile const uint16_t boci_notes[BOCI_LEN] = {25420, 20170, 25420, 20170, 16963, 16963, 25420, 20170, 25420, 20170, 16963, 16963, 12710, 13467, 15116, 16963, 19047, 15116, 16963, 19047, 20170, 22655, 25420, 25420};
volatile const uint8_t boci_beats[BOCI_LEN] = {6, 6, 6, 6, 4, 4, 6, 6, 6, 6, 4, 4, 6, 6, 6, 6, 4, 4, 6, 6, 6, 6, 4, 4};


/*
 * Betlti a hangjegynek megfelel szmot a timer compare regiszterbe
 * Loading the value referring to a specific note into the Timer Compare register
 */
void LoadNote(uint16_t note)
{
    //If the note is a "rest"
    if (note == 0)
    {
        //Turn off the output
        DDRD = (0 << 5);
    }

    else
    {
        //Enable output
        DDRD = (1 << 5);
        //Write the upper 8 bit
        OCR1AH = (note >> 8);
        //Write the lower 8 bit
        OCR1AL = note & 0xFF;
    }
}

/*
 * Own delay function, it is required for different beats
 */
void MyDelay(uint16_t time)
{
    uint16_t i;

    for (i = 0; i < time ; i++)
    {
        _delay_ms(1);
    }
}

/*
 * Playing the specified melody
 */
void Play(volatile const uint16_t* notes, volatile const uint8_t* beats, uint16_t length)
{
    uint8_t i = 0;
    uint8_t delay_between_notes;

    //Going through all elements of the arrays
    for (i = 0; i < length; i++)
    {
        //Determining the delay between each note
        delay_between_notes = SPEED / 100;
        
		//Load the next note from the array
        LoadNote(notes[i]);

        //The beat determines the duration of the note
        switch (beats[i])
        {
        case 0:
            MyDelay(SPEED);
            break;

        case 1:
            MyDelay(SPEED * 3 / 4);
            break;

        case 2:
            MyDelay(SPEED * 1 / 2);
            break;

        case 3:
            MyDelay(SPEED * 3 / 8);
            break;

        case 4:
            MyDelay(SPEED * 1 / 4);
            break;

        case 5:
            MyDelay(SPEED * 3 / 16);
            break;

        case 6:
            MyDelay(SPEED * 1 / 8);
            break;

        case 7:
            MyDelay(SPEED * 3 / 32);
            break;

        case 8:
            MyDelay(SPEED * 1 / 16);
            break;

        case 9:
            MyDelay(SPEED * 3 / 64);

        case 10:
            MyDelay(SPEED * 1 / 32);
            break;
        }

		//Creating a short "rest" after each note, so they will be heard as independent notes (legato) 
        LoadNote(0);
        MyDelay(delay_between_notes);
    }
}